@@ -75,17 +75,16 @@ void
 brasero_layout_add_source (BraseroLayout *layout,
 			   GtkWidget *child,
 			   const gchar *id,
-			   const gchar *name,
 			   const gchar *subtitle,
-			   const gchar *menu,
 			   const gchar *tooltip,
-			   const gchar *icon,
+			   const gchar *icon_name,
 			   BraseroLayoutType types);
 void
-brasero_layout_load (BraseroLayout *layout, BraseroLayoutType type);
+brasero_layout_load (BraseroLayout *layout,
+		     BraseroLayoutType type);
 
 void
-brasero_layout_register_menu (BraseroLayout *layout,
-			      GtkUIManager *manager);
+brasero_layout_register_ui (BraseroLayout *layout,
+			    GtkUIManager *manager);
 
 #endif /* BRASERO_LAYOUT_H */
