@@ -47,29 +47,29 @@
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
-#include "utils.h"
-#include "project.h"
+#include "brasero-utils.h"
+#include "brasero-project.h"
 #include "brasero-layout.h"
 #include "brasero-project-manager.h"
 #include "brasero-file-chooser.h"
 #include "brasero-uri-container.h"
-#include "burn-caps.h"
 #include "brasero-image-option-dialog.h"
-#include "burn-dialog.h"
-#include "project-type-chooser.h"
-#include "disc-copy-dialog.h"
+#include "brasero-burn-dialog.h"
+#include "brasero-project-type-chooser.h"
+#include "brasero-disc-copy-dialog.h"
 #include "brasero-vfs.h"
+#include "burn-caps.h"
 
 #ifdef BUILD_SEARCH
-#include "search.h"
+#include "brasero-search-beagle.h"
 #endif
 
 #ifdef BUILD_PLAYLIST
-#include "play-list.h"
+#include "brasero-playlist.h"
 #endif
 
 #ifdef BUILD_PREVIEW
-#include "player.h"
+#include "brasero-preview.h"
 #endif
 
 static void brasero_project_manager_class_init (BraseroProjectManagerClass *klass);
@@ -107,20 +107,20 @@ brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
 
 /* menus */
 static GtkActionEntry entries [] = {
-	{"New", GTK_STOCK_NEW, N_("New project"), NULL,
+	{"New", GTK_STOCK_NEW, N_("_New Project"), NULL,
 	 N_("Create a new project"), NULL },
-	{"NewChoose", GTK_STOCK_NEW, N_("Empty project"), NULL,
+	{"NewChoose", GTK_STOCK_NEW, N_("_Empty Project"), NULL,
 	 N_("Let you choose your new project"), G_CALLBACK (brasero_project_manager_new_empty_prj_cb)},
-	{"NewAudio", NULL, N_("New audio project"), NULL,
+	{"NewAudio", "media-optical-audio-new", N_("New _Audio Project"), NULL,
 	 N_("Create a new audio project"), G_CALLBACK (brasero_project_manager_new_audio_prj_cb)},
-	{"NewData", NULL, N_("New data project"), NULL,
+	{"NewData", "media-optical-data-new", N_("New _Data Project"), NULL,
 	 N_("Create a new data project"), G_CALLBACK (brasero_project_manager_new_data_prj_cb)},
-	{"NewCopy", NULL, N_("Copy disc"), NULL,
+	{"NewCopy", "media-optical-copy", N_("Copy _Disc..."), NULL,
 	 N_("Copy a disc"), G_CALLBACK (brasero_project_manager_new_copy_prj_cb)},
-	{"NewIso", NULL, N_("Burn an image"), NULL,
+	{"NewIso", "iso-image-burn", N_("_Burn Image..."), NULL,
 	 N_("Burn an image"), G_CALLBACK (brasero_project_manager_new_iso_prj_cb)},
 
-	{"Open", GTK_STOCK_OPEN, N_("_Open"), NULL,
+	{"Open", GTK_STOCK_OPEN, N_("_Open..."), NULL,
 	 N_("Open a project"), G_CALLBACK (brasero_project_manager_open_cb)},
 };
 
@@ -128,29 +128,24 @@ static const char *description = {
 	"<ui>"
 	    "<menubar name='menubar' >"
 		"<menu action='ProjectMenu'>"
-			"<placeholder name='ViewPlaceholder'/>"
-
-			"<menu action='New'>"
-				"<menuitem action='NewAudio'/>"
-				"<menuitem action='NewData'/>"
-				"<menuitem action='NewCopy'/>"	
-				"<menuitem action='NewIso'/>"	
+			"<placeholder name='ProjectPlaceholder'>"
+				"<placeholder name='ViewPlaceholder'/>"
+				"<menu action='New'>"
+					"<menuitem action='NewAudio'/>"
+					"<menuitem action='NewData'/>"
+					"<menuitem action='NewCopy'/>"	
+					"<menuitem action='NewIso'/>"	
 			"</menu>"
+			"</placeholder>"
 
-			"<separator/>"
-			"<placeholder name='ProjectPlaceholder'/>"
-			    "<menuitem action='Open'/>"
-			    "<menuitem action='Recent'/>"
+			"<placeholder name='ProjectPlaceholder'>"
 			    "<separator/>"
+			    "<menuitem action='Open'/>"
+			    "<menuitem action='RecentProjects'/>"
+			    "<separator/>"
+			"</placeholder>"
 		"</menu>"
 	    "</menubar>"
-
-	    "<toolbar name='toolbar'>"
-		"<toolitem action='NewChoose'/>"
-		"<toolitem action='Open'/>"
-		"<toolitem action='Save'/>"
-		"<separator/>"
-	    "</toolbar>"
 	"</ui>"
 };
 
@@ -163,6 +158,8 @@ struct BraseroProjectManagerPrivate {
 	GtkWidget *layout;
 	GtkWidget *status;
 
+	guint status_ctx;
+
 	GtkActionGroup *action_group;
 };
 
@@ -201,184 +198,6 @@ brasero_project_manager_get_type ()
 	return type;
 }
 
-static void
-brasero_project_manager_class_init (BraseroProjectManagerClass *klass)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS(klass);
-
-	parent_class = g_type_class_peek_parent(klass);
-	object_class->finalize = brasero_project_manager_finalize;
-}
-
-static void
-brasero_project_manager_init (BraseroProjectManager *obj)
-{
-	GtkWidget *type;
-	GtkWidget *scroll;
-	GtkWidget *chooser;
-
-	obj->priv = g_new0 (BraseroProjectManagerPrivate, 1);
-
-	gtk_notebook_set_show_border (GTK_NOTEBOOK (obj), FALSE);
-	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (obj), FALSE);
-
-	obj->priv->action_group = gtk_action_group_new ("ProjectManagerAction");
-	gtk_action_group_set_translation_domain (obj->priv->action_group, GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (obj->priv->action_group,
-				      entries,
-				      G_N_ELEMENTS (entries),
-				      obj);
-
-	/* add the project type chooser to the notebook */
-	type = brasero_project_type_chooser_new ();
-	gtk_widget_show (type);
-	scroll = gtk_scrolled_window_new (NULL, NULL);
-	gtk_widget_show (scroll);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
-					GTK_POLICY_AUTOMATIC,
-					GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scroll), type);
-
-	g_signal_connect (G_OBJECT (type),
-			  "chosen",
-			  G_CALLBACK (brasero_project_manager_type_changed_cb),
-			  obj);
-
-	gtk_notebook_prepend_page (GTK_NOTEBOOK (obj), scroll, NULL);
-
-	/* add the layout */
-	obj->priv->layout = brasero_layout_new ();
-	gtk_widget_show (obj->priv->layout);
-	gtk_notebook_append_page (GTK_NOTEBOOK (obj), obj->priv->layout, NULL);
-
-	/* create the project for audio and data discs */
-	obj->priv->project = brasero_project_new ();
-
-#ifdef BUILD_PREVIEW
-	GtkWidget *preview;
-
-	preview = brasero_player_new ();
-	gtk_widget_show (preview);
-	brasero_player_add_source (BRASERO_PLAYER (preview),
-				   BRASERO_URI_CONTAINER (obj->priv->project));
-
-#endif /* BUILD_PREVIEW */
-
-	chooser = brasero_file_chooser_new ();
-    	BRASERO_PROJECT_MANAGER_CONNECT_CHANGED (obj, chooser);
-
-	gtk_widget_show_all (chooser);
-	brasero_layout_add_source (BRASERO_LAYOUT (obj->priv->layout),
-				   chooser,
-				   "Chooser",
-				   _("<big><b>File Browser</b></big>"),
-				   _("<i>Browse the file system</i>"),
-				   _("File browser"),
-				   _("Display file browser"),
-				   GTK_STOCK_DIRECTORY,
-				   NULL,
-				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA);
-	brasero_project_add_source (BRASERO_PROJECT (obj->priv->project),
-				    BRASERO_URI_CONTAINER (chooser));
-
-#ifdef BUILD_PREVIEW
-	brasero_player_add_source (BRASERO_PLAYER (preview),
-				   BRASERO_URI_CONTAINER (chooser));
-#endif
-
-	brasero_layout_add_project (BRASERO_LAYOUT (obj->priv->layout),
-				    obj->priv->project);
-	gtk_widget_show_all (obj->priv->project);
-
-#ifdef BUILD_SEARCH
-	GtkWidget *search;
-
-	search = brasero_search_new ();
-    	BRASERO_PROJECT_MANAGER_CONNECT_CHANGED (obj, search);
-
-	gtk_widget_show_all (search);
-	brasero_layout_add_source (BRASERO_LAYOUT (obj->priv->layout),
-				   search,
-				   "Search",
-				   _("<big><b>Search Files</b></big>"),
-				   _("<i>Search files using keywords</i>"),
-				   _("Search files"),
-				   _("Display search"),
-				   GTK_STOCK_FIND,
-				   NULL,
-				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA);
-	brasero_project_add_source (BRASERO_PROJECT (obj->priv->project),
-				    BRASERO_URI_CONTAINER (search));
-
-#ifdef BUILD_PREVIEW
-	brasero_player_add_source (BRASERO_PLAYER (preview),
-				   BRASERO_URI_CONTAINER (search));
-#endif
-
-#endif /* BUILD_SEARCH */
-
-#ifdef BUILD_PLAYLIST
-	GtkWidget *playlist;
-
-	playlist = brasero_playlist_new ();
-    	BRASERO_PROJECT_MANAGER_CONNECT_CHANGED (obj, playlist);
-	gtk_widget_show_all (playlist);
-	brasero_layout_add_source (BRASERO_LAYOUT (obj->priv->layout),
-				   playlist,
-				   "Playlist",
-				   _("<big><b>Playlists</b></big>"),
-				   _("<i>Display playlists and their contents</i>"),
-				   _("Playlists"),
-				   _("Display playlists"),
-				   NULL,
-				   "audio-x-generic", 
-				   BRASERO_LAYOUT_AUDIO);
-
-	brasero_project_add_source (BRASERO_PROJECT (obj->priv->project),
-				    BRASERO_URI_CONTAINER (playlist));
-
-#ifdef BUILD_PREVIEW
-	brasero_player_add_source (BRASERO_PLAYER (preview),
-				   BRASERO_URI_CONTAINER (playlist));
-#endif
-
-#endif /* BUILD_PLAYLIST */
-
-#ifdef BUILD_PREVIEW
-	brasero_layout_add_preview (BRASERO_LAYOUT (obj->priv->layout),
-				    preview);
-#endif
-	
-}
-
-static void
-brasero_project_manager_finalize (GObject *object)
-{
-	BraseroProjectManager *cobj;
-
-	cobj = BRASERO_PROJECT_MANAGER (object);
-
-	if (cobj->priv->vfs) {
-		brasero_vfs_cancel (cobj->priv->vfs, object);
-		g_object_unref (cobj->priv->vfs);
-		cobj->priv->vfs = NULL;
-	}
-
-	g_free (cobj->priv);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
-GtkWidget *
-brasero_project_manager_new ()
-{
-	BraseroProjectManager *obj;
-	
-	obj = BRASERO_PROJECT_MANAGER (g_object_new (BRASERO_TYPE_PROJECT_MANAGER, NULL));
-	
-	return GTK_WIDGET (obj);
-}
-
 static void
 brasero_project_manager_size_preview (BraseroVFS *vfs,
 				      GObject *object,
@@ -391,7 +210,8 @@ brasero_project_manager_size_preview (BraseroVFS *vfs,
 	gint valid_num = files_num - invalid_num;
     	gchar *status_string = NULL;
 
-	gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status), 1);
+	gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status),
+			   manager->priv->status_ctx);
 
 	if (!invalid_num && valid_num) {
 		gchar *size_string;
@@ -444,7 +264,7 @@ brasero_project_manager_size_preview (BraseroVFS *vfs,
 		status_string = g_strdup (_("No file selected"));
 
 	gtk_statusbar_push (GTK_STATUSBAR (manager->priv->status),
-			    1,
+			    manager->priv->status_ctx,
 			    status_string);
 	g_free (status_string);
 }
@@ -463,9 +283,10 @@ brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
 
 	uris = brasero_uri_container_get_selected_uris (container);
     	if (!uris) {
-		gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status), 1);
+		gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status),
+				   manager->priv->status_ctx);
 		gtk_statusbar_push (GTK_STATUSBAR (manager->priv->status),
-				    1,
+				    manager->priv->status_ctx,
 				    _("No file selected"));
 		return;
 	}
@@ -495,38 +316,29 @@ brasero_project_manager_set_status (BraseroProjectManager *manager,
 				    GtkWidget *status)
 {
 	manager->priv->status = status;
+	manager->priv->status_ctx = gtk_statusbar_get_context_id (GTK_STATUSBAR (status),
+								  "size_info");
 }
 
 void
-brasero_project_manager_register_menu (BraseroProjectManager *manager,
-				       GtkUIManager *ui_manager)
+brasero_project_manager_register_ui (BraseroProjectManager *manager,
+				     GtkUIManager *ui_manager)
 {
-	GtkAction *action;
 	GError *error = NULL;
 
-	action = gtk_action_new ("Recent",
-				 _("Recent projects"),
-				 _("Display the projects recently opened"),
-				 NULL);
-	gtk_action_group_add_action (manager->priv->action_group, action);
-	g_object_unref (action);
-
 	gtk_ui_manager_insert_action_group (ui_manager, manager->priv->action_group, 0);
 	if (!gtk_ui_manager_add_ui_from_string (ui_manager, description, -1, &error)) {
-		g_message ("building menus failed: %s", error->message);
+		g_message ("building UI failed: %s", error->message);
 		g_error_free (error);
 	}
-   	brasero_layout_register_menu (BRASERO_LAYOUT (manager->priv->layout), ui_manager);
-	brasero_project_register_menu (BRASERO_PROJECT (manager->priv->project), ui_manager);
+
+	brasero_project_register_ui (BRASERO_PROJECT (manager->priv->project), ui_manager);
+   	brasero_layout_register_ui (BRASERO_LAYOUT (manager->priv->layout), ui_manager);
 }
 
 static void
 brasero_project_manager_burn (BraseroProjectManager *manager,
-			      NautilusBurnDrive *drive,
-			      gint speed,
-			      gchar *output,
-			      const BraseroTrackSource *source,
-			      BraseroBurnFlag flags)
+			      BraseroBurnSession *session)
 {
 	GtkWidget *toplevel;
 	GtkWidget *dialog;
@@ -543,13 +355,7 @@ brasero_project_manager_burn (BraseroProjectManager *manager,
 	gtk_widget_show_all (dialog);
 
 	destroy = brasero_burn_dialog_run (BRASERO_BURN_DIALOG (dialog),
-					   drive,
-					   speed,
-					   output,
-					   source,
-					   flags,
-					   0,
-					   FALSE);
+					   session);
 
 	gtk_widget_destroy (dialog);
 
@@ -569,14 +375,10 @@ static void
 brasero_project_manager_burn_iso_dialog (BraseroProjectManager *manager,
 					 const gchar *uri)
 {
-	BraseroBurnFlag flags = BRASERO_BURN_FLAG_NONE;
-	BraseroTrackSource *track;
-	NautilusBurnDrive *drive;
+	BraseroBurnSession *session;
 	GtkResponseType result;
-	gchar *output = NULL;
 	GtkWidget *toplevel;
 	GtkWidget *dialog;
-	gint speed;
 
 	/* setup, show, and run options dialog */
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
@@ -595,38 +397,23 @@ brasero_project_manager_burn_iso_dialog (BraseroProjectManager *manager,
 		return;
 	}
 
-	brasero_image_option_dialog_get_param (BRASERO_IMAGE_OPTION_DIALOG (dialog),
-					       &flags,
-					       &drive,
-					       &speed,
-					       &track);
+	session = brasero_image_option_dialog_get_session (BRASERO_IMAGE_OPTION_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 
-	brasero_project_manager_burn (manager,
-				      drive,
-				      speed,
-				      output,
-				      track,
-				      flags);
+	if (!session)
+		return;
 
-	brasero_track_source_free (track);
-	nautilus_burn_drive_unref (drive);
-
-	if (output)
-		g_free (output);
+	brasero_project_manager_burn (manager, session);
+	g_object_unref (session);
 }
 
 static void
 brasero_project_manager_burn_disc (BraseroProjectManager *manager)
 {
-	BraseroTrackSource *source;
-	NautilusBurnDrive *drive;
+	BraseroBurnSession *session;
 	GtkResponseType result;
-	BraseroBurnFlag flags;
-	gchar *output = NULL;
 	GtkWidget *toplevel;
 	GtkWidget *dialog;
-	gint speed;
 
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
 
@@ -642,21 +429,11 @@ brasero_project_manager_burn_disc (BraseroProjectManager *manager)
 		return;
 	}
 
-	brasero_disc_copy_dialog_get_session_param (BRASERO_DISC_COPY_DIALOG (dialog),
-						    &drive,
-						    &speed,
-						    &output,
-						    &source,
-						    &flags);
+	session = brasero_disc_copy_dialog_get_session (BRASERO_DISC_COPY_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 
-	brasero_project_manager_burn (manager,
-				      drive,
-				      speed,
-				      output,
-				      source,
-				      flags);
-	brasero_track_source_free (source);
+	brasero_project_manager_burn (manager, session);
+	g_object_unref (session);
 }
 
 static void
@@ -669,9 +446,16 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 	GtkWidget *toplevel;
 	GtkAction *action;
 
+	if ((manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO
+	||   manager->priv->type == BRASERO_PROJECT_TYPE_DATA)
+	&&  !brasero_project_confirm_switch (BRASERO_PROJECT (manager->priv->project)))
+		return;
+
+	gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status),
+			   manager->priv->status_ctx);
+
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
 	action = gtk_action_group_get_action (manager->priv->action_group, "NewChoose");
-	gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status), 1);
 
 	manager->priv->type = type;
 
@@ -829,10 +613,19 @@ brasero_project_manager_iso (BraseroProjectManager *manager, const gchar *uri)
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_ISO, NULL, uri, TRUE);
 }
 
+static void
+brasero_project_manager_uri_clicked_cb (BraseroProjectTypeChooser *chooser,
+					const gchar *uri,
+					BraseroProjectManager *manager)
+{
+	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_ISO, NULL, uri, TRUE);
+}
+
 void
 brasero_project_manager_open (BraseroProjectManager *manager, const gchar *uri)
 {
 	BraseroProjectType type;
+	GtkAction *action;
 
     	gtk_widget_show (manager->priv->layout);
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (manager), 1);
@@ -840,12 +633,26 @@ brasero_project_manager_open (BraseroProjectManager *manager, const gchar *uri)
 
 	manager->priv->type = type;
 
-    	if (type == BRASERO_PROJECT_TYPE_INVALID)
+    	if (type == BRASERO_PROJECT_TYPE_INVALID) {
 		brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
-	else if (type == BRASERO_PROJECT_TYPE_DATA)
+		return;
+	}
+
+	if (type == BRASERO_PROJECT_TYPE_DATA)
 		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_DATA);
 	else
 		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_AUDIO);
+
+	action = gtk_action_group_get_action (manager->priv->action_group, "NewChoose");
+	gtk_action_set_sensitive (action, TRUE);
+}
+
+static void
+brasero_project_manager_project_clicked_cb (BraseroProjectTypeChooser *chooser,
+					    const gchar *uri,
+					    BraseroProjectManager *manager)
+{
+	brasero_project_manager_open (manager, uri);
 }
 
 void
@@ -856,12 +663,8 @@ brasero_project_manager_empty (BraseroProjectManager *manager)
 
 gboolean
 brasero_project_manager_load_session (BraseroProjectManager *manager,
-				      const gchar *path,
-				      gint position)
+				      const gchar *path)
 {
-    	if (position > 0)
-		gtk_paned_set_position (GTK_PANED (manager->priv->layout), position);
-
     	if (path) {
 		gchar *uri;
 		BraseroProjectType type;
@@ -880,12 +683,9 @@ brasero_project_manager_load_session (BraseroProjectManager *manager,
 gboolean
 brasero_project_manager_save_session (BraseroProjectManager *manager,
 				      const gchar *path,
-				      gint *position)
+				      gboolean cancellable)
 {
-    	gboolean result = TRUE;
-
-    	if (position)
-		*position = gtk_paned_get_position (GTK_PANED (manager->priv->layout));
+    	gboolean result = FALSE;
 
     	if (path) {
 		gchar *uri;
@@ -895,9 +695,182 @@ brasero_project_manager_save_session (BraseroProjectManager *manager,
 	 	 * DataDisc has not finished exploration */
 		uri = gnome_vfs_make_uri_from_input (path);
     		result = brasero_project_save_session (BRASERO_PROJECT (manager->priv->project),
-						       uri);
+						       uri,
+						       cancellable);
 		g_free (uri);
 	}
 
     	return result;
 }
+
+static void
+brasero_project_manager_init (BraseroProjectManager *obj)
+{
+	GtkWidget *type;
+	GtkAction *action;
+	GtkWidget *chooser;
+
+	obj->priv = g_new0 (BraseroProjectManagerPrivate, 1);
+
+	gtk_notebook_set_show_border (GTK_NOTEBOOK (obj), FALSE);
+	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (obj), FALSE);
+
+	obj->priv->action_group = gtk_action_group_new ("ProjectManagerAction");
+	gtk_action_group_set_translation_domain (obj->priv->action_group, GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (obj->priv->action_group,
+				      entries,
+				      G_N_ELEMENTS (entries),
+				      obj);
+
+	action = gtk_action_group_get_action (obj->priv->action_group, "NewChoose");
+	g_object_set (action,
+		      "short-label", _("New"), /* for toolbar buttons */
+		      NULL);
+	action = gtk_action_group_get_action (obj->priv->action_group, "Open");
+	g_object_set (action,
+		      "short-label", _("Open"), /* for toolbar buttons */
+		      NULL);
+
+	/* add the project type chooser to the notebook */
+	type = brasero_project_type_chooser_new ();
+	gtk_widget_show (type);
+	g_signal_connect (type,
+			  "chosen",
+			  G_CALLBACK (brasero_project_manager_type_changed_cb),
+			  obj);
+	g_signal_connect (type,
+			  "uri-clicked",
+			  G_CALLBACK (brasero_project_manager_uri_clicked_cb),
+			  obj);
+	g_signal_connect (type,
+			  "project-clicked",
+			  G_CALLBACK (brasero_project_manager_project_clicked_cb),
+			  obj);
+	gtk_notebook_prepend_page (GTK_NOTEBOOK (obj), type, NULL);
+
+	/* add the layout */
+	obj->priv->layout = brasero_layout_new ();
+	gtk_widget_show (obj->priv->layout);
+	gtk_notebook_append_page (GTK_NOTEBOOK (obj), obj->priv->layout, NULL);
+
+	/* create the project for audio and data discs */
+	obj->priv->project = brasero_project_new ();
+
+#ifdef BUILD_PREVIEW
+
+	GtkWidget *preview;
+
+	preview = brasero_preview_new ();
+	gtk_widget_show (preview);
+	brasero_preview_add_source (BRASERO_PREVIEW (preview),
+				    BRASERO_URI_CONTAINER (obj->priv->project));
+
+#endif /* BUILD_PREVIEW */
+
+	chooser = brasero_file_chooser_new ();
+    	BRASERO_PROJECT_MANAGER_CONNECT_CHANGED (obj, chooser);
+
+	gtk_widget_show_all (chooser);
+	brasero_layout_add_source (BRASERO_LAYOUT (obj->priv->layout),
+				   chooser,
+				   "Chooser",
+				   _("Browse the file system"),
+				   _("Display file browser"),
+				   GTK_STOCK_DIRECTORY,
+				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA);
+
+#ifdef BUILD_PREVIEW
+	brasero_preview_add_source (BRASERO_PREVIEW (preview),
+				    BRASERO_URI_CONTAINER (chooser));
+#endif
+
+	brasero_layout_add_project (BRASERO_LAYOUT (obj->priv->layout),
+				    obj->priv->project);
+	gtk_widget_show_all (obj->priv->project);
+
+#ifdef BUILD_SEARCH
+	GtkWidget *search;
+
+	search = brasero_search_new ();
+    	BRASERO_PROJECT_MANAGER_CONNECT_CHANGED (obj, search);
+
+	gtk_widget_show_all (search);
+	brasero_layout_add_source (BRASERO_LAYOUT (obj->priv->layout),
+				   search,
+				   "Search",
+				   _("Search files using keywords"),
+				   _("Display search"),
+				   GTK_STOCK_FIND,
+				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA);
+
+#ifdef BUILD_PREVIEW
+	brasero_preview_add_source (BRASERO_PREVIEW (preview),
+				    BRASERO_URI_CONTAINER (search));
+#endif
+
+#endif /* BUILD_SEARCH */
+
+#ifdef BUILD_PLAYLIST
+	GtkWidget *playlist;
+
+	playlist = brasero_playlist_new ();
+    	BRASERO_PROJECT_MANAGER_CONNECT_CHANGED (obj, playlist);
+	gtk_widget_show_all (playlist);
+	brasero_layout_add_source (BRASERO_LAYOUT (obj->priv->layout),
+				   playlist,
+				   "Playlist",
+				   _("Display playlists and their contents"),
+				   _("Display playlists"),
+				   "audio-x-generic", 
+				   BRASERO_LAYOUT_AUDIO);
+
+#ifdef BUILD_PREVIEW
+	brasero_preview_add_source (BRASERO_PREVIEW (preview),
+				    BRASERO_URI_CONTAINER (playlist));
+#endif
+
+#endif /* BUILD_PLAYLIST */
+
+#ifdef BUILD_PREVIEW
+	brasero_layout_add_preview (BRASERO_LAYOUT (obj->priv->layout),
+				    preview);
+#endif
+	
+}
+
+static void
+brasero_project_manager_finalize (GObject *object)
+{
+	BraseroProjectManager *cobj;
+
+	cobj = BRASERO_PROJECT_MANAGER (object);
+
+	if (cobj->priv->vfs) {
+		brasero_vfs_cancel (cobj->priv->vfs, object);
+		g_object_unref (cobj->priv->vfs);
+		cobj->priv->vfs = NULL;
+	}
+
+	g_free (cobj->priv);
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+static void
+brasero_project_manager_class_init (BraseroProjectManagerClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS(klass);
+
+	parent_class = g_type_class_peek_parent(klass);
+	object_class->finalize = brasero_project_manager_finalize;
+}
+
+GtkWidget *
+brasero_project_manager_new ()
+{
+	BraseroProjectManager *obj;
+	
+	obj = BRASERO_PROJECT_MANAGER (g_object_new (BRASERO_TYPE_PROJECT_MANAGER, NULL));
+	
+	return GTK_WIDGET (obj);
+}
