@@ -67,20 +67,25 @@ brasero_project_manager_open (BraseroProjectManager *manager, const gchar *uri);
 void
 brasero_project_manager_empty (BraseroProjectManager *manager);
 
+/**
+ * returns TRUE on error to try to stop app closing
+ */
+
 gboolean
 brasero_project_manager_save_session (BraseroProjectManager *manager,
 				      const gchar *path,
-				      gint *pos);
+				      gboolean cancellable);
 gboolean
 brasero_project_manager_load_session (BraseroProjectManager *manager,
-				      const gchar *path,
-				      gint pos);
+				      const gchar *path);
 
 void
-brasero_project_manager_register_menu (BraseroProjectManager *manager,
-				       GtkUIManager *ui_manager);
+brasero_project_manager_register_ui (BraseroProjectManager *manager,
+				     GtkUIManager *ui_manager);
 void
 brasero_project_manager_set_status (BraseroProjectManager *manager,
 				    GtkWidget *status);
 
+G_END_DECLS
+
 #endif /* BRASERO_PROJECT_MANAGER_H */
