@@ -49,18 +49,29 @@ brasero_session_load (BraseroApp *app, gboolean load_project)
 	gchar *height_str = NULL;
 	gchar *width_str = NULL;
 	gchar *state_str = NULL;
-	gchar *pane_str = NULL;
 	gchar *version = NULL;
     	gchar *project_path;
-	gint pane_pos = -1;
-	gint height = 480;
-	gint width = 640;
+	gint height;
+	gint width;
 	gint state = 0;
 
 	gchar *session_path;
 	xmlNodePtr item;
 	xmlDocPtr session = NULL;
 
+	GdkScreen *screen;
+	GdkRectangle rect;
+	gint monitor;
+
+	/* Make sure that on first run the window has a default size of at least
+	 * 85% of the screen (hardware not GTK+) */
+	screen = gtk_window_get_screen (GTK_WINDOW (app->mainwin));
+	monitor = gdk_screen_get_monitor_at_window (screen, 
+						    GTK_WIDGET (app->mainwin)->window);
+	gdk_screen_get_monitor_geometry (screen, monitor, &rect);
+	width = rect.width / 100 * 85;
+	height = rect.height / 100 * 85;
+
 	session_path = gnome_util_home_file (BRASERO_SESSION_TMP_SESSION_PATH);
 	if (!session_path)
 		goto end;
@@ -112,14 +123,6 @@ brasero_session_load (BraseroApp *app, gboolean load_project)
 								   item->xmlChildrenNode,
 								   1);
 		}
-		else if (!xmlStrcmp (item->name, (const xmlChar *) "pane")) {
-			if (pane_str)
-				goto end;
-
-			pane_str = (char *) xmlNodeListGetString (session,
-								  item->xmlChildrenNode,
-								  1);
-		}
 		else if (item->type == XML_ELEMENT_NODE)
 			goto end;
 
@@ -139,9 +142,6 @@ brasero_session_load (BraseroApp *app, gboolean load_project)
 	if (state_str)
 		state = (int) g_strtod (state_str, NULL);
 
-	if (pane_str)
-		pane_pos = (int) g_strtod (pane_str, NULL);
-
 end:
 	if (height_str)
 		g_free (height_str);
@@ -152,9 +152,6 @@ end:
 	if (state_str)
 		g_free (state_str);
 
-	if (pane_str)
-		g_free (pane_str);
-
 	if (version)
 		g_free (version);
 
@@ -176,8 +173,7 @@ end:
 	}
 
     	brasero_project_manager_load_session (BRASERO_PROJECT_MANAGER (app->contents),
-					      project_path,
-					      pane_pos);
+					      project_path);
 
     	if (project_path) {
     		/* remove the project file not to have it next time */
@@ -189,11 +185,13 @@ end:
 }
 
 gboolean
-brasero_session_save (BraseroApp *app, gboolean save_project)
+brasero_session_save (BraseroApp *app,
+		      gboolean save_project,
+		      gboolean cancellable)
 {
 	gint success;
-	gint pane_pos;
-    	gchar *project_path;
+	gboolean cancel;
+	gchar *project_path;
 	gchar *session_path;
 	xmlTextWriter *session;
 
@@ -202,11 +200,16 @@ brasero_session_save (BraseroApp *app, gboolean save_project)
     	else
 		project_path = NULL;
 
-    	brasero_project_manager_save_session (BRASERO_PROJECT_MANAGER (app->contents),
-					      project_path,
-					      &pane_pos);
+    	cancel = brasero_project_manager_save_session (BRASERO_PROJECT_MANAGER (app->contents),
+						       project_path,
+						       cancellable);
     	g_free (project_path);
 
+	/* see if we should try to cancel shutdown */
+	if (cancel)
+		return TRUE;
+
+	/* now save the state of the window */
 	session_path = gnome_util_home_file (BRASERO_SESSION_TMP_SESSION_PATH);
 	if (!session_path)
 		return FALSE;
@@ -260,14 +263,6 @@ brasero_session_save (BraseroApp *app, gboolean save_project)
 	if (success < 0)
 		goto error;
 
-	/* saves internal pane geometry */
-	success = xmlTextWriterWriteFormatElement (session,
-						   (xmlChar *) "pane",
-						   "%i",
-						   pane_pos);
-	if (success < 0)
-		goto error;
-
 	success = xmlTextWriterEndElement (session);
 	if (success < 0)
 		goto error;
@@ -276,7 +271,7 @@ brasero_session_save (BraseroApp *app, gboolean save_project)
 	xmlFreeTextWriter (session);
 	g_free (session_path);
 
-	return TRUE;
+	return FALSE;
 
 error:
 	xmlTextWriterEndDocument (session);
@@ -291,7 +286,7 @@ static void
 brasero_session_die_cb (GnomeClient *client_loc,
 			BraseroApp *app)
 {
-	brasero_session_save (app, FALSE);
+	brasero_session_save (app, FALSE, FALSE);
 	gtk_widget_destroy (app->mainwin);
 }
 
@@ -308,7 +303,7 @@ brasero_session_save_yourself_cb (GnomeClient *client_loc,
     	gchar *argv [] = { 	"brasero",
 				NULL	};
 
-    	brasero_session_save (app, TRUE);
+    	brasero_session_save (app, TRUE, FALSE);
 	gnome_client_set_clone_command (client_loc,
 					argc,
 					argv);
