@@ -74,9 +74,9 @@ const gchar *types [] = {	N_("file"),
 				N_("DVD+RW dual layer"),
 				N_("DVD-R dual layer"),
 				N_("DVD-RAM"),
-				N_("blue ray disc"),
-				N_("writable blue ray disc"),
-				N_("rewritable blue ray disc"),
+				N_("Blu-ray disc"),
+				N_("Writable Blu-ray disc"),
+				N_("Rewritable Blu-ray disc"),
 				NULL };
 
 
@@ -100,16 +100,25 @@ struct _BraseroMediumPrivate
 	gint64 block_size;
 
 	guint64 next_wr_add;
-	BraseroMediumInfo info;
+	BraseroMedia info;
 	NautilusBurnDrive * drive;
 };
 
 #define BRASERO_MEDIUM_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_MEDIUM, BraseroMediumPrivate))
 
+/**
+ * Try to open the drive exclusively but don't block; if drive can't be opened
+ * exclusively then retry every second until we're shut or the drive state
+ * changes to not busy.
+ * No exclusive at the moment since when the medium is mounted we can't use excl
+ */
+
+#define OPEN_FLAGS			O_RDONLY /*|O_EXCL */|O_NONBLOCK
+#define BUSY_RETRY_TIME			1000
+
 enum
 {
 	PROP_0,
-
 	PROP_DRIVE
 };
 
@@ -133,7 +142,7 @@ brasero_medium_get_icon (BraseroMedium *medium)
 	return priv->icon;
 }
 
-BraseroMediumInfo
+BraseroMedia
 brasero_medium_get_status (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
@@ -151,8 +160,10 @@ brasero_medium_get_tracks (BraseroMedium *medium)
 	return g_slist_copy (priv->tracks);
 }
 
-gint64
-brasero_medium_get_last_data_track_address (BraseroMedium *medium)
+gboolean
+brasero_medium_get_last_data_track_address (BraseroMedium *medium,
+					    gint64 *byte,
+					    gint64 *sector)
 {
 	GSList *iter;
 	BraseroMediumPrivate *priv;
@@ -168,9 +179,159 @@ brasero_medium_get_last_data_track_address (BraseroMedium *medium)
 			track = current;
 	}
 
-	if (!track)
-		return -1;
-	return track->start;
+	if (!track) {
+		if (byte)
+			*byte = -1;
+		if (sector)
+			*sector = -1;
+		return FALSE;
+	}
+
+	if (byte)
+		*byte = track->start * priv->block_size;
+
+	if (sector)
+		*sector = track->start;
+
+	return TRUE;
+}
+
+gboolean
+brasero_medium_get_last_data_track_space (BraseroMedium *medium,
+					  gint64 *size,
+					  gint64 *blocks)
+{
+	GSList *iter;
+	BraseroMediumPrivate *priv;
+	BraseroMediumTrack *track = NULL;
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+
+	for (iter = priv->tracks; iter; iter = iter->next) {
+		BraseroMediumTrack *current;
+
+		current = iter->data;
+		if (current->type & BRASERO_MEDIUM_TRACK_DATA)
+			track = current;
+	}
+
+	if (!track) {
+		if (size)
+			*size = -1;
+		if (blocks)
+			*blocks = -1;
+		return FALSE;
+	}
+
+	if (size)
+		*size = track->blocks_num * priv->block_size;
+	if (blocks)
+		*blocks = track->blocks_num;
+
+	return TRUE;
+}
+
+guint
+brasero_medium_get_track_num (BraseroMedium *medium)
+{
+	guint retval = 0;
+	GSList *iter;
+	BraseroMediumPrivate *priv;
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+	for (iter = priv->tracks; iter; iter = iter->next) {
+		BraseroMediumTrack *current;
+
+		current = iter->data;
+		if (current->type & BRASERO_MEDIUM_TRACK_LEADOUT)
+			break;
+
+		retval ++;
+	}
+
+	return retval;
+}
+
+static BraseroMediumTrack *
+brasero_medium_get_track (BraseroMedium *medium,
+			  guint num)
+{
+	guint i = 1;
+	GSList *iter;
+	BraseroMediumPrivate *priv;
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+
+	for (iter = priv->tracks; iter; iter = iter->next) {
+		BraseroMediumTrack *current;
+
+		current = iter->data;
+		if (current->type == BRASERO_MEDIUM_TRACK_LEADOUT)
+			break;
+
+		if (i == num)
+			return current;
+
+		i++;
+	}
+
+	return NULL;
+}
+
+gboolean
+brasero_medium_get_track_space (BraseroMedium *medium,
+				guint num,
+				gint64 *size,
+				gint64 *blocks)
+{
+	BraseroMediumPrivate *priv;
+	BraseroMediumTrack *track;
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+
+	track = brasero_medium_get_track (medium, num);
+	if (!track) {
+		if (size)
+			*size = -1;
+		if (blocks)
+			*blocks = -1;
+		return FALSE;
+	}
+
+	if (size)
+		*size = track->blocks_num * priv->block_size;
+	if (blocks)
+		*blocks = track->blocks_num;
+
+	return TRUE;
+}
+
+gboolean
+brasero_medium_get_track_address (BraseroMedium *medium,
+				  guint num,
+				  gint64 *byte,
+				  gint64 *sector)
+{
+	BraseroMediumPrivate *priv;
+	BraseroMediumTrack *track;
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+
+	track = brasero_medium_get_track (medium, num);
+	if (!track) {
+		if (byte)
+			*byte = -1;
+		if (sector)
+			*sector = -1;
+		return FALSE;
+	}
+
+	if (byte)
+		*byte = track->start * priv->block_size;
+	if (sector)
+		*sector = track->start;
+
+	return TRUE;	
 }
 
 gint64
@@ -182,13 +343,13 @@ brasero_medium_get_next_writable_address (BraseroMedium *medium)
 	return priv->next_wr_add;
 }
 
-gint
+gint64
 brasero_medium_get_max_write_speed (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
 
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
-	return priv->max_wrt;
+	return priv->max_wrt * 1024;
 }
 
 /**
@@ -226,6 +387,17 @@ brasero_medium_get_data_size (BraseroMedium *medium,
 
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
+	if (!priv->tracks) {
+		/* that's probably because it wasn't possible to retrieve info */
+		if (size)
+			*size = 0;
+
+		if (blocks)
+			*blocks = 0;
+
+		return;
+	}
+
 	for (iter = priv->tracks; iter; iter = iter->next) {
 		BraseroMediumTrack *tmp;
 
@@ -237,10 +409,10 @@ brasero_medium_get_data_size (BraseroMedium *medium,
 	}
 
 	if (size)
-		*size = track ? (track->start + track->blocks_num) * priv->block_size: -1;
+		*size = track ? (track->start + track->blocks_num) * priv->block_size: 0;
 
 	if (blocks)
-		*blocks = track ? track->start + track->blocks_num: -1;
+		*blocks = track ? track->start + track->blocks_num: 0;
 }
 
 void
@@ -254,6 +426,28 @@ brasero_medium_get_free_space (BraseroMedium *medium,
 
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
+	if (!priv->tracks) {
+		/* that's probably because it wasn't possible to retrieve info.
+		 * maybe it also happens with unformatted DVD+RW */
+
+		if (priv->info & BRASERO_MEDIUM_CLOSED) {
+			if (size)
+				*size = 0;
+
+			if (blocks)
+				*blocks = 0;
+		}
+		else {
+			if (size)
+				*size = priv->block_num * priv->block_size;
+
+			if (blocks)
+				*blocks = priv->block_num;
+		}
+
+		return;
+	}
+
 	for (iter = priv->tracks; iter; iter = iter->next) {
 		BraseroMediumTrack *tmp;
 
@@ -264,11 +458,29 @@ brasero_medium_get_free_space (BraseroMedium *medium,
 		}
 	}
 
-	if (size)
-		*size = track ? track->blocks_num * priv->block_size: -1;
+	if (size) {
+		if (!track) {
+			/* No leadout was found so the disc is probably closed:
+			 * no free space left. */
+			*size = 0;
+		}
+		else if (track->blocks_num <= 0)
+			*size = (priv->block_num - track->start) * priv->block_size;
+		else
+			*size = track->blocks_num * priv->block_size;
+	}
 
-	if (blocks)
-		*blocks = track ? track->blocks_num: -1;
+	if (blocks) {
+		if (!track) {
+			/* No leadout was found so the disc is probably closed:
+			 * no free space left. */
+			*blocks = 0;
+		}
+		else if (track->blocks_num <= 0)
+			*blocks = priv->block_num - track->blocks_num;
+		else
+			*blocks = track->blocks_num;
+	}
 }
 
 void
@@ -287,7 +499,7 @@ brasero_medium_get_capacity (BraseroMedium *medium,
 		if (blocks)
 			*blocks = priv->block_num;
 	}
-	else  if (!(priv->info & BRASERO_MEDIUM_WRITABLE))
+	else  if (priv->info & BRASERO_MEDIUM_CLOSED)
 		brasero_medium_get_data_size (medium, size, blocks);
 	else
 		brasero_medium_get_free_space (medium, size, blocks);
@@ -302,25 +514,42 @@ brasero_medium_get_capacity_CD_RW (BraseroMedium *self,
 				   int fd,
 				   BraseroScsiErrCode *code)
 {
-	BraseroScsiAtipData atip_data;
+	BraseroScsiAtipData *atip_data = NULL;
 	BraseroMediumPrivate *priv;
 	BraseroScsiResult result;
+	int size = 0;
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
+	BRASERO_BURN_LOG ("Retrieving capacity from atip");
+
 	result = brasero_mmc1_read_atip (fd,
 					 &atip_data,
-					 sizeof (atip_data),
+					 &size,
 					 NULL);
 
 	if (result != BRASERO_SCSI_OK) {
-		BRASERO_BURN_LOG ("READ ATIP failed");
+		BRASERO_BURN_LOG ("READ ATIP failed (scsi error)");
 		return BRASERO_BURN_ERR;
 	}
 
-	priv->block_num = BRASERO_MSF_TO_LBA (atip_data.desc->leadout_mn,
-					      atip_data.desc->leadout_sec,
-					      atip_data.desc->leadout_frame);
+	/* check the size of the structure: it must be at least 16 bytes long */
+	if (size < 16) {
+		if (size)
+			g_free (atip_data);
+
+		BRASERO_BURN_LOG ("READ ATIP failed (wrong size)");
+		return BRASERO_BURN_ERR;
+	}
+
+	priv->block_num = BRASERO_MSF_TO_LBA (atip_data->desc->leadout_mn,
+					      atip_data->desc->leadout_sec,
+					      atip_data->desc->leadout_frame);
+	g_free (atip_data);
+
+	BRASERO_BURN_LOG ("Format capacity %lli %lli",
+			  priv->block_num,
+			  priv->block_size);
 
 	return BRASERO_BURN_OK;
 }
@@ -336,6 +565,8 @@ brasero_medium_get_capacity_DVD_RW (BraseroMedium *self,
 	BraseroScsiResult result;
 	gint size;
 
+	BRASERO_BURN_LOG ("Retrieving format capacity");
+
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 	result = brasero_mmc2_read_format_capacities (fd,
 						      &hdr,
@@ -366,6 +597,10 @@ brasero_medium_get_capacity_DVD_RW (BraseroMedium *self,
 				if (desc->format_type == BRASERO_SCSI_DVDRW_PLUS) {
 					priv->block_num = BRASERO_GET_32 (desc->blocks_num);
 					priv->block_size = BRASERO_GET_24 (desc->type_param);
+
+					/* that can happen */
+					if (!priv->block_size)
+						priv->block_size = 2048;
 					break;
 				}
 			}
@@ -381,6 +616,10 @@ brasero_medium_get_capacity_DVD_RW (BraseroMedium *self,
 		priv->block_size = BRASERO_GET_24 (current->block_size);
 	}
 
+	BRASERO_BURN_LOG ("Format capacity %lli %lli",
+			  priv->block_num,
+			  priv->block_size);
+
 	g_free (hdr);
 	return BRASERO_BURN_OK;
 }
@@ -424,6 +663,8 @@ brasero_medium_get_speed_mmc3 (BraseroMedium *self,
 	BraseroScsiWrtSpdDesc *desc;
 	BraseroScsiGetPerfData *wrt_perf = NULL;
 
+	BRASERO_BURN_LOG ("Retrieving speed (Get Performance)");
+
 	/* NOTE: this only work if there is RT streaming feature with
 	 * wspd bit set to 1. At least an MMC3 drive. */
 	priv = BRASERO_MEDIUM_PRIVATE (self);
@@ -436,7 +677,7 @@ brasero_medium_get_speed_mmc3 (BraseroMedium *self,
 		g_free (wrt_perf);
 
 		BRASERO_BURN_LOG ("GET PERFORMANCE failed");
-		return BRASERO_BURN_RETRY;
+		return BRASERO_BURN_ERR;
 	}
 
 	num_desc = (size - sizeof (BraseroScsiGetPerfHdr)) /
@@ -471,7 +712,7 @@ end:
 	 * function but don't report any speed. So if our top speed is 0 then
 	 * use the other way to get the speed. It was a Teac */
 	if (!priv->max_wrt)
-		return BRASERO_BURN_RETRY;
+		return BRASERO_BURN_ERR;
 
 	return BRASERO_BURN_OK;
 }
@@ -491,6 +732,8 @@ brasero_medium_get_page_2A_write_speed_desc (BraseroMedium *self,
 	gint max_num;
 	int size = 0;
 
+	BRASERO_BURN_LOG ("Retrieving speed (2A speeds)");
+
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 	result = brasero_spc1_mode_sense_get_page (fd,
 						   BRASERO_SPC_PAGE_STATUS,
@@ -506,6 +749,7 @@ brasero_medium_get_page_2A_write_speed_desc (BraseroMedium *self,
 
 	page_2A = (BraseroScsiStatusPage *) &data->page;
 
+	/* FIXME: the following is not necessarily true */
 	if (size < sizeof (BraseroScsiStatusPage)) {
 		g_free (data);
 
@@ -554,6 +798,8 @@ brasero_medium_get_page_2A_max_speed (BraseroMedium *self,
 	BraseroScsiResult result;
 	int size = 0;
 
+	BRASERO_BURN_LOG ("Retrieving speed (2A max)");
+
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
 	result = brasero_spc1_mode_sense_get_page (fd,
@@ -590,43 +836,80 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 				BraseroScsiErrCode *code)
 {
 	BraseroScsiGetConfigHdr *hdr = NULL;
-	BraseroScsiRTStreamDesc *stream;
 	BraseroMediumPrivate *priv;
 	BraseroScsiResult result;
 	int size;
 
+	BRASERO_BURN_LOG ("Retrieving media profile");
+
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 	result = brasero_mmc2_get_configuration_feature (fd,
 							 BRASERO_SCSI_FEAT_REAL_TIME_STREAM,
 							 &hdr,
 							 &size,
 							 code);
-	if (result == BRASERO_SCSI_INVALID_COMMAND) {
-		g_free (hdr);
+	if (result != BRASERO_SCSI_OK) {
+		BraseroScsiAtipData *data = NULL;
+		int size = 0;
 
 		BRASERO_BURN_LOG ("GET CONFIGURATION failed");
 
-		/* This is probably a MMC1 drive since this command was
+		/* This could be a MMC1 drive since this command was
 		 * introduced in MMC2 and is supported onward. So it
 		 * has to be a CD (R/RW). The rest of the information
 		 * will be provided by read_disc_information. */
-		priv->info = BRASERO_MEDIUM_CD;
 
-		result = brasero_medium_get_page_2A_max_speed (self, fd, code);
+		/* The only thing here left to determine is if that's a WRITABLE
+		 * or a REWRITABLE. To determine that information, we need to
+		 * read TocPmaAtip. It if fails that's a ROM, if it succeeds.
+		 * No need to set error code since we consider that it's a ROM
+		 * if a failure happens. */
+		result = brasero_mmc1_read_atip (fd,
+						 &data,
+						 &size,
+						 NULL);
+		if (result != BRASERO_SCSI_OK) {
+			/* CDROM */
+			priv->info = BRASERO_MEDIUM_CDROM;
+			priv->type = types [1];
+			priv->icon = icons [1];
+		}
+		else {
+			/* check the size of the structure: it must be at least 8 bytes long */
+			if (size < 8) {
+				if (size)
+					g_free (data);
+
+				BRASERO_BURN_LOG ("READ ATIP failed (wrong size)");
+				return BRASERO_BURN_ERR;
+			}
+
+			if (data->desc->erasable) {
+				/* CDRW */
+				priv->info = BRASERO_MEDIUM_CDRW;
+				priv->type = types [3];
+				priv->icon = icons [3];
+			}
+			else {
+				/* CDR */
+				priv->info = BRASERO_MEDIUM_CDR;
+				priv->type = types [2];
+				priv->icon = icons [2];
+			}
+
+			g_free (data);
+		}
+
+		/* retrieve the speed */
+		result = brasero_medium_get_page_2A_max_speed (self,
+							       fd,
+							       code);
 		return result;
 	}
 
-	if (result != BRASERO_SCSI_OK) {
-		/* All other commands means an error */
-		g_free (hdr);
-
-		BRASERO_BURN_LOG ("GET CONFIGURATION failed");
-		return BRASERO_BURN_ERR;
-	}
-
 	switch (BRASERO_GET_16 (hdr->current_profile)) {
 	case BRASERO_SCSI_PROF_CDROM:
-		priv->info = BRASERO_MEDIUM_CD;
+		priv->info = BRASERO_MEDIUM_CDROM;
 		priv->type = types [1];
 		priv->icon = icons [1];
 		break;
@@ -644,7 +927,7 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 		break;
 
 	case BRASERO_SCSI_PROF_DVD_ROM:
-		priv->info = BRASERO_MEDIUM_DVD;
+		priv->info = BRASERO_MEDIUM_DVD_ROM;
 		priv->type = types [4];
 		priv->icon = icons [4];
 		break;
@@ -679,6 +962,7 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 		priv->icon = icons [7];
 		break;
 
+	/* WARNING: these types are recognized, no more */
 	case BRASERO_SCSI_PROF_DVD_R_PLUS_DL:
 		priv->info = BRASERO_MEDIUM_DVDR_PLUS_DL;
 		priv->type = types [9];
@@ -745,43 +1029,39 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 	case BRASERO_SCSI_PROF_HD_DVD_R:
 	case BRASERO_SCSI_PROF_HD_DVD_RAM:
 		priv->info = BRASERO_MEDIUM_UNSUPPORTED;
+		priv->icon = icons [0];
 		g_free (hdr);
 		return BRASERO_BURN_NOT_SUPPORTED;
 	}
 
-	BRASERO_BURN_LOG ("medium is %d", priv->info);
-
-	/* see how we should get the speeds */
-	if (hdr->desc->add_len != sizeof (BraseroScsiRTStreamDesc)) {
-		g_free (hdr);
-
-		BRASERO_BURN_LOG ("wrong size for Stream descriptor");
-		return BRASERO_BURN_ERR;
-	}
-
 	/* try all SCSI functions to get write/read speeds in order */
-	stream = (BraseroScsiRTStreamDesc *) hdr->desc->data;
-	if (stream->wrt_spd) {
-		/* NOTE: the next function returns either OK or RETRY to make 
-		 * sure we always go and get over an error. There are other ways
-		 * to get the information we want */
-		result = brasero_medium_get_speed_mmc3 (self, fd, code);
-		if (result != BRASERO_BURN_RETRY)
-			goto end;
+	if (hdr->desc->add_len >= sizeof (BraseroScsiRTStreamDesc)) {
+		BraseroScsiRTStreamDesc *stream;
+
+		/* means it's at least an MMC3 drive */
+		stream = (BraseroScsiRTStreamDesc *) hdr->desc->data;
+		if (stream->wrt_spd) {
+			result = brasero_medium_get_speed_mmc3 (self, fd, code);
+			if (result == BRASERO_BURN_OK)
+				goto end;
+		}
+
+		if (stream->mp2a) {
+			result = brasero_medium_get_page_2A_write_speed_desc (self, fd, code);
+			if (result == BRASERO_BURN_OK)
+				goto end;
+		}
 	}
 
-	if (stream->mp2a)
-		result = brasero_medium_get_page_2A_write_speed_desc (self, fd, code);
-	else
-		result = brasero_medium_get_page_2A_max_speed (self, fd, code);
-
+	/* fallback for speeds */
+	result = brasero_medium_get_page_2A_max_speed (self, fd, code);
 
 end:
 
 	g_free (hdr);
 
 	if (result != BRASERO_BURN_OK)
-		return BRASERO_BURN_ERR;
+		return result;
 
 	return BRASERO_BURN_OK;
 }
@@ -797,6 +1077,8 @@ brasero_medium_get_css_feature (BraseroMedium *self,
 	int size;
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	BRASERO_BURN_LOG ("Testing for Css encrypted media");
 	result = brasero_mmc2_get_configuration_feature (fd,
 							 BRASERO_SCSI_FEAT_DVD_CSS,
 							 &hdr,
@@ -809,14 +1091,16 @@ brasero_medium_get_css_feature (BraseroMedium *self,
 		return BRASERO_BURN_ERR;
 	}
 
-	if (hdr->desc->add_len != sizeof (BraseroScsiDVDCssDesc)) {
+	if (hdr->desc->add_len < sizeof (BraseroScsiDVDCssDesc)) {
 		g_free (hdr);
-		return BRASERO_BURN_ERR;
+		return BRASERO_BURN_OK;
 	}
 
 	/* here we just need to see if this feature is current or not */
-	if (hdr->desc->current)
+	if (hdr->desc->current) {
 		priv->info |= BRASERO_MEDIUM_PROTECTED;
+		BRASERO_BURN_LOG ("media is Css protected");
+	}
 
 	g_free (hdr);
 	return BRASERO_BURN_OK;
@@ -826,12 +1110,36 @@ brasero_medium_get_css_feature (BraseroMedium *self,
  * Functions to get information about disc contents
  */
 
-/**
- * NOTE: for DVD-R multisession we lose 28688 blocks for each session
- * so the capacity is the addition of all session sizes + 28688 for each
- * For all multisession DVD-/+R and CDR-RW the remaining size is given 
- * in the leadout. One exception though with DVD+/-RW.
- */
+static void
+brasero_medium_set_track_type (BraseroMedium *self,
+			       BraseroMediumTrack *track,
+			       guchar control)
+{
+	BraseroMediumPrivate *priv;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	if (control & BRASERO_SCSI_TRACK_COPY)
+		track->type |= BRASERO_MEDIUM_TRACK_COPY;
+
+	if (!(control & BRASERO_SCSI_TRACK_DATA)) {
+		track->type |= BRASERO_MEDIUM_TRACK_AUDIO;
+		priv->info |= BRASERO_MEDIUM_HAS_AUDIO;
+
+		if (control & BRASERO_SCSI_TRACK_PREEMP)
+			track->type |= BRASERO_MEDIUM_TRACK_PREEMP;
+
+		if (control & BRASERO_SCSI_TRACK_4_CHANNELS)
+			track->type |= BRASERO_MEDIUM_TRACK_4_CHANNELS;
+	}
+	else {
+		track->type |= BRASERO_MEDIUM_TRACK_DATA;
+		priv->info |= BRASERO_MEDIUM_HAS_DATA;
+
+		if (control & BRASERO_SCSI_TRACK_DATA_INCREMENTAL)
+			track->type |= BRASERO_MEDIUM_TRACK_INCREMENTAL;
+	}
+}
 
 static BraseroBurnResult
 brasero_medium_track_volume_size (BraseroMedium *self,
@@ -840,6 +1148,7 @@ brasero_medium_track_volume_size (BraseroMedium *self,
 {
 	BraseroMediumPrivate *priv;
 	BraseroBurnResult res;
+	GError *error = NULL;
 	gint64 nb_blocks;
 
 	if (!track)
@@ -860,8 +1169,15 @@ brasero_medium_track_volume_size (BraseroMedium *self,
 					  track->start,
 					  &nb_blocks,
 					  NULL);
-	if (!res)
+	if (!res) {
+		BRASERO_BURN_LOG ("Failed to retrieve the volume size: %s",
+				  error && error->message ? 
+				  error->message:"unknown error");
+
+		if (error)
+			g_error_free (error);
 		return BRASERO_BURN_ERR;
+	}
 
 	track->blocks_num = nb_blocks;
 	return BRASERO_BURN_OK;
@@ -877,13 +1193,26 @@ brasero_medium_track_get_info (BraseroMedium *self,
 	BraseroScsiTrackInfo track_info;
 	BraseroMediumPrivate *priv;
 	BraseroScsiResult result;
+	int size;
+
+	BRASERO_BURN_LOG ("Retrieving track information for %i", track_num);
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
+	/* at this point we know the type of the disc that's why we set the 
+	 * size according to this type. That may help to avoid outrange address
+	 * errors. */
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DL|BRASERO_MEDIUM_WRITABLE))
+		size = 48;
+	else if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_PLUS|BRASERO_MEDIUM_WRITABLE))
+		size = 40;
+	else
+		size = 36;
+
 	result = brasero_mmc1_read_track_info (fd,
 					       track_num,
 					       &track_info,
-					       sizeof (BraseroScsiTrackInfo),
+					       &size,
 					       code);
 
 	if (result != BRASERO_SCSI_OK) {
@@ -892,6 +1221,7 @@ brasero_medium_track_get_info (BraseroMedium *self,
 	}
 
 	track->blocks_num = BRASERO_GET_32 (track_info.track_size);
+	track->session = BRASERO_SCSI_SESSION_NUM (track_info);
 
 	/* Now here is a potential bug: we can write tracks (data or not)
 	 * shorter than 300 Kio /2 sec but they will be padded to reach this
@@ -901,15 +1231,390 @@ brasero_medium_track_get_info (BraseroMedium *self,
 	 * use the old value.
 	 * That's important for checksuming to have a perfect account of the 
 	 * data size. */
-	if (track->blocks_num == 300)
+	if (track->blocks_num <= 300) {
+		BRASERO_BURN_LOG ("300 sectors size. Checking for real size");
 		brasero_medium_track_volume_size (self, track, fd);
+	}
 
 	if (track_info.next_wrt_address_valid)
 		priv->next_wr_add = BRASERO_GET_32 (track_info.next_wrt_address);
 
+	BRASERO_BURN_LOG ("Track %i (session %i): type = %i start = %llu size = %llu",
+			  track_num,
+			  track->session,
+			  track->type,
+			  track->start,
+			  track->blocks_num);
+
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * return :
+ *  0 when it's not possible to determine (fallback to formatted toc)
+ * -1 for BCD
+ *  1 for HEX */
+static guint
+brasero_medium_check_BCD_use (BraseroMedium *self,
+			      int fd,
+			      BraseroScsiRawTocDesc *desc,
+			      guint num,
+			      BraseroScsiErrCode *code)
+{
+	guint i;
+	int size;
+	guint leadout = 0;
+	guint track_num = 0;
+	gboolean use_BCD = TRUE;
+	gboolean use_HEX = TRUE;
+	BraseroScsiResult result;
+	BraseroScsiTrackInfo track_info;
+	guint start_BCD, start_LBA, track_start;
+
+	/* first check if all values are valid BCD numbers in the descriptors */
+	for (i = 0; i < num; i++) {
+		if (desc [i].adr == 1 && desc [i].point <= BRASERO_SCSI_Q_SUB_CHANNEL_TRACK_START) {
+			if (!BRASERO_IS_BCD_VALID (desc [i].p_min)
+			||  !BRASERO_IS_BCD_VALID (desc [i].p_sec)
+			||  !BRASERO_IS_BCD_VALID (desc [i].p_frame)) {
+				use_BCD = FALSE;
+				break;
+			}
+		}
+		else if (desc [i].point == BRASERO_SCSI_Q_SUB_CHANNEL_LEADOUT_START) {
+			if (!BRASERO_IS_BCD_VALID (desc [i].p_min)
+			||  !BRASERO_IS_BCD_VALID (desc [i].p_sec)
+			||  !BRASERO_IS_BCD_VALID (desc [i].p_frame)) {
+				use_BCD = FALSE;
+				break;
+			}
+		}
+	}
+
+	/* then check if there are valid Hex values */
+	for (i = 0; i < num; i++) {
+		if (desc [i].adr != 1 || desc [i].point > BRASERO_SCSI_Q_SUB_CHANNEL_TRACK_START)
+			continue;
+
+		if (desc [i].p_min > 99
+		||  desc [i].p_sec > 59
+		||  desc [i].p_frame > 74) {
+			use_HEX = FALSE;
+			break;
+		}
+	}
+
+	if (use_BCD != use_HEX) {
+		if (use_BCD)
+			return -1;
+
+		return 1;
+	}
+
+	/* To check if the drive uses BCD values or HEX values we ask for the 
+	 * track information that contains also the start for the track but in
+	 * HEX values. If values are the same then it works. */
+
+	/* NOTE: there could be another way to do it: get first track, in LBA
+	 * and BCD it must be 150. */
+
+	/* First find the first track and get track start address in BCD */
+	BRASERO_BURN_LOG ("Retrieving track information to determine number format");
+
+	for (i = 0; i < num; i++) {
+		if (desc [i].adr == BRASERO_SCSI_Q_SUB_CHANNEL_LEADIN_MODE5
+		&&  desc [i].point == BRASERO_SCSI_Q_SUB_CHANNEL_MULTI_NEXT_SESSION) {
+			/* store the leadout number just in case */
+			leadout = i;
+			continue;
+		}
+
+		if (desc [i].adr != 1 || desc [i].point > BRASERO_SCSI_Q_SUB_CHANNEL_TRACK_START)
+			continue;
+
+		track_num ++;
+
+		start_BCD = BRASERO_MSF_TO_LBA (BRASERO_GET_BCD (desc [i].p_min),
+						BRASERO_GET_BCD (desc [i].p_sec),
+						BRASERO_GET_BCD (desc [i].p_frame));
+
+		start_LBA = BRASERO_MSF_TO_LBA (desc [i].p_min,
+						desc [i].p_sec,
+						desc [i].p_frame);
+
+		BRASERO_BURN_LOG ("Comparing to track information from READ TRACK INFO for track %i", track_num);
+
+		size = 36;
+		start_LBA -= 150;
+		start_BCD -= 150;
+
+		result = brasero_mmc1_read_track_info (fd,
+						       track_num,
+						       &track_info,
+						       &size,
+						       code);
+
+		if (result != BRASERO_SCSI_OK) {
+			BRASERO_BURN_LOG ("READ TRACK INFO failed");
+			/* Fallback to formatted toc */
+			return 0;
+		}
+
+		track_start = BRASERO_GET_32 (track_info.start_lba);
+		BRASERO_BURN_LOG ("comparing DCB %i and LBA %i to real start address %i",
+				  start_BCD, start_LBA, track_start);
+
+		/* try to find a conclusive match */
+		if (track_start == start_BCD && track_start != start_LBA)
+			return -1;
+
+		if (track_start == start_LBA && track_start != start_BCD)
+			return 1;
+	}
+
+	/* Our last chance, the leadout.
+	 * NOTE: no need to remove 150 sectors here. */
+	start_BCD = BRASERO_MSF_TO_LBA (BRASERO_GET_BCD (desc [leadout].min),
+					BRASERO_GET_BCD (desc [leadout].sec),
+					BRASERO_GET_BCD (desc [leadout].frame));
+
+	start_LBA = BRASERO_MSF_TO_LBA (desc [leadout].min,
+					desc [leadout].sec,
+					desc [leadout].frame);
+
+	BRASERO_BURN_LOG ("Comparing to track information from READ TRACK INFO for leadout");
+
+	size = 36;
+
+	/* leadout number is number of tracks + 1 */
+	result = brasero_mmc1_read_track_info (fd,
+					       track_num + 1,
+					       &track_info,
+					       &size,
+					       code);
+
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_BURN_LOG ("READ TRACK INFO failed for leadout");
+		/* Fallback to formatted toc */
+		return 0;
+	}
+
+	track_start = BRASERO_GET_32 (track_info.start_lba);
+	BRASERO_BURN_LOG ("comparing DCB %i and LBA %i to real start address %i",
+			  start_BCD, start_LBA, track_start);
+
+	/* try to find a conclusive match */
+	if (track_start == start_BCD && track_start != start_LBA)
+		return -1;
+
+	if (track_start == start_LBA && track_start != start_BCD)
+		return 1;
+
+	/* fallback to formatted toc */
+	return 0;
+}
+
+/**
+ * The reason why we use this perhaps more lengthy method is that with
+ * multisession discs, the first track is reported to be two sectors shorter
+ * than it should. As I don't know why and since the following works we use
+ * this one. */
+static BraseroBurnResult
+brasero_medium_get_CD_sessions_info (BraseroMedium *self,
+				     int fd,
+				     BraseroScsiErrCode *code)
+{
+	gint use_bcd;
+	GSList *iter;
+	int num, i, size;
+	gint leadout_start = 0;
+	BraseroScsiResult result;
+	BraseroMediumPrivate *priv;
+	BraseroScsiRawTocDesc *desc;
+	BraseroScsiRawTocData *toc = NULL;
+
+	BRASERO_BURN_LOG ("Reading Raw Toc");
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	size = 0;
+	result = brasero_mmc1_read_toc_raw (fd,
+					    0,
+					    &toc,
+					    &size,
+					    code);
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_BURN_LOG ("READ TOC failed");
+		return BRASERO_BURN_ERR;
+	}
+
+	num = (size - sizeof (BraseroScsiRawTocData)) /
+	       sizeof (BraseroScsiRawTocDesc);
+
+	BRASERO_BURN_LOG ("%i track(s) found", num);
+
+	desc = toc->desc;
+	use_bcd = brasero_medium_check_BCD_use (self, fd, desc, num, code);
+	if (!use_bcd) {
+		g_free (toc);
+
+		BRASERO_BURN_LOG ("Fallback to formatted toc");
+		return BRASERO_BURN_ERR;
+	}
+
+	if (use_bcd > 0)
+		use_bcd = 0;
+
+	if (use_bcd) {
+		BRASERO_BURN_LOG ("Using BCD format");
+	}
+	else {
+		BRASERO_BURN_LOG ("Using HEX format");
+	}
+
+	for (i = 0; i < num; i++, desc ++) {
+		BraseroMediumTrack *track;
+
+		track = NULL;
+		if (desc->adr == 1 && desc->point <= BRASERO_SCSI_Q_SUB_CHANNEL_TRACK_START) {
+			track = g_new0 (BraseroMediumTrack, 1);
+			track->session = desc->session_num;
+
+			brasero_medium_set_track_type (self, track, desc->control);
+			if (use_bcd)
+				track->start = BRASERO_MSF_TO_LBA (BRASERO_GET_BCD (desc->p_min),
+								   BRASERO_GET_BCD (desc->p_sec),
+								   BRASERO_GET_BCD (desc->p_frame));
+			else
+				track->start = BRASERO_MSF_TO_LBA (desc->p_min,
+								   desc->p_sec,
+								   desc->p_frame);
+
+			track->start -= 150;
+
+			/* if there are tracks and the last previously added track is in
+			 * the same session then set the size */
+			if (priv->tracks) {
+				BraseroMediumTrack *last_track;
+
+				last_track = priv->tracks->data;
+				if (last_track->session == track->session)
+					last_track->blocks_num = track->start - last_track->start;
+			}
+
+			priv->tracks = g_slist_prepend (priv->tracks, track);
+		}
+		else if (desc->point == BRASERO_SCSI_Q_SUB_CHANNEL_LEADOUT_START) {
+			/* NOTE: the leadout session is first in the list. So if
+			 * we have tracks in the list set the last session track
+			 * size when we reach a new leadout (and therefore a new
+			 * session). */
+
+			if (priv->tracks) {
+				BraseroMediumTrack *last_track;
+
+				last_track = priv->tracks->data;
+				last_track->blocks_num = leadout_start - last_track->start;
+			}
+
+			if (use_bcd)
+				leadout_start = BRASERO_MSF_TO_LBA (BRASERO_GET_BCD (desc->p_min),
+								    BRASERO_GET_BCD (desc->p_sec),
+								    BRASERO_GET_BCD (desc->p_frame));
+			else
+				leadout_start = BRASERO_MSF_TO_LBA (desc->p_min,
+								    desc->p_sec,
+								    desc->p_frame);
+			leadout_start -= 150;
+		}
+	}
+
+	if (priv->tracks) {
+		BraseroMediumTrack *last_track;
+
+		/* set the last found track size */
+		last_track = priv->tracks->data;
+		last_track->blocks_num = leadout_start - last_track->start;
+	}
+
+	/* Add a leadout */
+	if (!(priv->info & BRASERO_MEDIUM_CLOSED)) {
+		BraseroMediumTrack *track;
+
+		/* we shouldn't request info on leadout if the disc is closed */
+		track = g_new0 (BraseroMediumTrack, 1);
+		priv->tracks = g_slist_prepend (priv->tracks, track);
+		track->start = leadout_start;
+		track->type = BRASERO_MEDIUM_TRACK_LEADOUT;
+
+		brasero_medium_track_get_info (self, track, g_slist_length (priv->tracks), fd, code); 
+	}
+
+	priv->tracks = g_slist_reverse (priv->tracks);
+
+	for (iter = priv->tracks; iter; iter = iter->next) {
+		BraseroMediumTrack *track;
+
+		track = iter->data;
+
+		/* check for tracks less that 300 sectors */
+		if (track->blocks_num <= 300 && track->type != BRASERO_MEDIUM_TRACK_LEADOUT) {
+			BRASERO_BURN_LOG ("300 sectors size. Checking for real size");
+			brasero_medium_track_volume_size (self, track, fd);
+		}
+
+		BRASERO_BURN_LOG ("Track %i: type = %i start = %llu size = %llu",
+				  g_slist_index (priv->tracks, track),
+				  track->type,
+				  track->start,
+				  track->blocks_num);
+	}
+
+	g_free (toc);
+	return BRASERO_BURN_OK;
+}
+
+/**
+ * NOTE: for DVD-R multisession we lose 28688 blocks for each session
+ * so the capacity is the addition of all session sizes + 28688 for each
+ * For all multisession DVD-/+R and CDR-RW the remaining size is given 
+ * in the leadout. One exception though with DVD+/-RW.
+ */
+
+static void
+brasero_medium_add_DVD_plus_RW_leadout (BraseroMedium *self,
+					gint32 start)
+{
+	BraseroMediumTrack *leadout;
+	BraseroMediumPrivate *priv;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	leadout = g_new0 (BraseroMediumTrack, 1);
+	priv->tracks = g_slist_append (priv->tracks, leadout);
+
+	leadout->start = start;
+	leadout->type = BRASERO_MEDIUM_TRACK_LEADOUT;
+
+	/* we fabricate the leadout here. We don't really need one in 
+	 * fact since it is always at the last sector whatever the
+	 * amount of data written. So we need in fact to read the file
+	 * system and get the last sector from it. Hopefully it won't be
+	 * buggy */
+	priv->next_wr_add = 0;
+
+	leadout->blocks_num = priv->block_num;
+	if (g_slist_length (priv->tracks) > 1) {
+		BraseroMediumTrack *track;
+
+		track = priv->tracks->data;
+		leadout->blocks_num -= ((track->blocks_num > 300) ? track->blocks_num : 300);
+	}
+	BRASERO_BURN_LOG ("Adding fabricated leadout start = %llu length = %llu",
+			  leadout->start,
+			  leadout->blocks_num);
+}
+
 static BraseroBurnResult
 brasero_medium_get_sessions_info (BraseroMedium *self,
 				  int fd,
@@ -921,6 +1626,8 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 	BraseroMediumPrivate *priv;
 	BraseroScsiFormattedTocData *toc = NULL;
 
+	BRASERO_BURN_LOG ("Reading Toc");
+
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 	result = brasero_mmc1_read_toc_formatted (fd,
 						  0,
@@ -937,6 +1644,8 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 	num = (size - sizeof (BraseroScsiFormattedTocData)) /
 	       sizeof (BraseroScsiTocDesc);
 
+	BRASERO_BURN_LOG ("%i track(s) found", num);
+
 	desc = toc->desc;
 	for (i = 0; i < num; i ++, desc ++) {
 		BraseroMediumTrack *track;
@@ -970,17 +1679,17 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 		}
 		else if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
 		     ||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED)) {
-				BraseroBurnResult result;
+			BraseroBurnResult result;
 
-			/* a special case for these two kinds of media
+			/* a special case for these two kinds of media (DVD+RW)
 			 * which have only one track: the first. */
 			result = brasero_medium_track_volume_size (self, 
 								   track,
 								   fd);
 			if (result == BRASERO_BURN_OK) {
 				track->type |= BRASERO_MEDIUM_TRACK_DATA;
-				priv->info |= BRASERO_MEDIUM_APPENDABLE|
-					      BRASERO_MEDIUM_HAS_DATA;
+				priv->info |= BRASERO_MEDIUM_HAS_DATA;
+
 				priv->next_wr_add = 0;
 
 				if (desc->control & BRASERO_SCSI_TRACK_DATA_INCREMENTAL)
@@ -991,6 +1700,7 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 				g_free (track);
 
 				priv->info |= BRASERO_MEDIUM_BLANK;
+				priv->info &= ~BRASERO_MEDIUM_CLOSED;
 			}
 		}
 		else {
@@ -1002,12 +1712,19 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 		}
 	}
 
-	/* we shouldn't request info on leadout if the disc is closed */
-	if (priv->info & (BRASERO_MEDIUM_APPENDABLE|BRASERO_MEDIUM_BLANK)) {
+	/* put the tracks in the right order */
+	priv->tracks = g_slist_reverse (priv->tracks);
+
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
+	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED))
+		brasero_medium_add_DVD_plus_RW_leadout (self, BRASERO_GET_32 (desc->track_start));
+	else if (!(priv->info & BRASERO_MEDIUM_CLOSED)) {
 		BraseroMediumTrack *track;
 
+		/* we shouldn't request info on leadout if the disc is closed
+		 * (except for DVD+/- (restricted) RW (see above) */
 		track = g_new0 (BraseroMediumTrack, 1);
-		priv->tracks = g_slist_prepend (priv->tracks, track);
+		priv->tracks = g_slist_append (priv->tracks, track);
 		track->start = BRASERO_GET_32 (desc->track_start);
 		track->type = BRASERO_MEDIUM_TRACK_LEADOUT;
 
@@ -1018,29 +1735,6 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 					       code);
 	}
 
-	/* put the tracks in the right order */
-	priv->tracks = g_slist_reverse (priv->tracks);
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
-	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED)) {
-		GSList *node;
-		BraseroMediumTrack *leadout;
-
-		node = g_slist_last (priv->tracks);
-		leadout = node->data;
-
-		if (g_slist_length (priv->tracks) > 1) {
-			BraseroMediumTrack *track;
-
-			track = priv->tracks->data;
-			leadout->blocks_num -= track->blocks_num;
-		}
-		else {
-			leadout->blocks_num = leadout->start;
-			leadout->start = 0;
-		}
-	}
-
 	g_free (toc);
 
 	return BRASERO_BURN_OK;
@@ -1056,6 +1750,8 @@ brasero_medium_get_contents (BraseroMedium *self,
 	BraseroMediumPrivate *priv;
 	BraseroScsiDiscInfoStd *info = NULL;
 
+	BRASERO_BURN_LOG ("Retrieving media status");
+
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
 	result = brasero_mmc1_read_disc_information_std (fd,
@@ -1075,21 +1771,46 @@ brasero_medium_get_contents (BraseroMedium *self,
 	if (info->status == BRASERO_SCSI_DISC_EMPTY) {
 		BraseroMediumTrack *track;
 
+		BRASERO_BURN_LOG ("Empty media");
+
 		priv->info |= BRASERO_MEDIUM_BLANK;
 		priv->block_size = 2048;
 
-		track = g_new0 (BraseroMediumTrack, 1);
-		track->start = 0;
-		track->type = BRASERO_MEDIUM_TRACK_LEADOUT;
-		priv->tracks = g_slist_prepend (priv->tracks, track);
-		brasero_medium_track_get_info (self, track, 1, fd, code);
+		if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
+		||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED))
+			brasero_medium_add_DVD_plus_RW_leadout (self, 0);
+		else {
+			track = g_new0 (BraseroMediumTrack, 1);
+			track->start = 0;
+			track->type = BRASERO_MEDIUM_TRACK_LEADOUT;
+			priv->tracks = g_slist_prepend (priv->tracks, track);
+			
+			brasero_medium_track_get_info (self,
+						       track,
+						       1,
+						       fd,
+						       code);
+		}
 		goto end;
 	}
 
-	if (info->status == BRASERO_SCSI_DISC_INCOMPLETE)
+	if (info->status == BRASERO_SCSI_DISC_INCOMPLETE) {
 		priv->info |= BRASERO_MEDIUM_APPENDABLE;
+		BRASERO_BURN_LOG ("Appendable media");
+	}
+	else if (info->status == BRASERO_SCSI_DISC_FINALIZED) {
+		priv->info |= BRASERO_MEDIUM_CLOSED;
+		BRASERO_BURN_LOG ("Closed media");
+	}
+
+	if (priv->info & BRASERO_MEDIUM_CD) {
+		result = brasero_medium_get_CD_sessions_info (self, fd, code);
+		if (result != BRASERO_BURN_OK)
+			result = brasero_medium_get_sessions_info (self, fd, code);
+	}
+	else
+		result = brasero_medium_get_sessions_info (self, fd, code);
 
-	result = brasero_medium_get_sessions_info (self, fd, code);
 	if (result != BRASERO_BURN_OK)
 		goto end;
 
@@ -1102,27 +1823,33 @@ end:
 static void
 brasero_medium_init_real (BraseroMedium *object, int fd)
 {
+	gchar *name;
 	BraseroBurnResult result;
 	BraseroMediumPrivate *priv;
 	BraseroScsiErrCode code = 0;
 
 	priv = BRASERO_MEDIUM_PRIVATE (object);
 
+	name = nautilus_burn_drive_get_name_for_display (priv->drive);
+	BRASERO_BURN_LOG ("Initializing information for medium in %s", name);
+	g_free (name);
+
 	result = brasero_medium_get_medium_type (object, fd, &code);
 	if (result != BRASERO_BURN_OK)
 		return;
 
-	if (priv->info & BRASERO_MEDIUM_DVD) {
-		result = brasero_medium_get_css_feature (object, fd, &code);
-		if (result != BRASERO_BURN_OK)
-			return;
-	}
+	brasero_medium_get_capacity_by_type (object, fd, &code);
 
 	result = brasero_medium_get_contents (object, fd, &code);
 	if (result != BRASERO_BURN_OK)
 		return;
 
-	brasero_medium_get_capacity_by_type (object, fd, &code);
+	/* assume that css feature is only for DVD-ROM which might be wrong but
+	 * some drives wrongly reports that css is enabled for blank DVD+R/W */
+	if (BRASERO_MEDIUM_IS (priv->info, (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_ROM)))
+		brasero_medium_get_css_feature (object, fd, &code);
+
+	BRASERO_BURN_LOG_DISC_TYPE (priv->info, "media is ");
 }
 
 static gboolean
@@ -1138,7 +1865,7 @@ brasero_medium_retry_open (gpointer object)
 	path = nautilus_burn_drive_get_device (priv->drive);
 
 	BRASERO_BURN_LOG ("Retrying to open device %s", path);
-	fd = open (path, O_RDONLY|O_NONBLOCK);
+	fd = open (path, OPEN_FLAGS);
 	if (fd < 0) {
 		if (errno == EBUSY
 		||  errno == EAGAIN
@@ -1156,6 +1883,8 @@ brasero_medium_retry_open (gpointer object)
 
 	BRASERO_BURN_LOG ("Open () succeeded\n");
 	priv->info = BRASERO_MEDIUM_NONE;
+	priv->icon = icons [0];
+
 	priv->retry_id = 0;
 
 	brasero_medium_init_real (self, fd);
@@ -1177,14 +1906,16 @@ brasero_medium_try_open (BraseroMedium *self)
 	/* the drive might be busy (a burning is going on) so we don't block
 	 * but we re-try to open it every second */
 	BRASERO_BURN_LOG ("Trying to open device %s", path);
-	fd = open (path, O_RDONLY|O_NONBLOCK);
+	fd = open (path, OPEN_FLAGS);
 	if (fd < 0) {
 		if (errno == EAGAIN
 		||  errno == EWOULDBLOCK
 		||  errno == EBUSY) {
 			BRASERO_BURN_LOG ("Device busy");
 			priv->info = BRASERO_MEDIUM_BUSY;
-			priv->retry_id = g_timeout_add (1000,
+			priv->icon = icons [0];
+
+			priv->retry_id = g_timeout_add (BUSY_RETRY_TIME,
 							brasero_medium_retry_open,
 							self);
 		}
