@@ -40,11 +40,8 @@ G_BEGIN_DECLS
 #define BRASERO_IS_PROCESS_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), BRASERO_TYPE_PROCESS))
 #define BRASERO_PROCESS_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), BRASERO_TYPE_PROCESS, BraseroProcessClass))
 
-typedef struct BraseroProcessPrivate BraseroProcessPrivate;
-
 typedef struct {
 	BraseroJob parent;
-	BraseroProcessPrivate *priv;
 } BraseroProcess;
 
 typedef struct {
@@ -57,12 +54,22 @@ typedef struct {
 						 const gchar *line);
 	BraseroBurnResult	(*set_argv)	(BraseroProcess *process,
 						 GPtrArray *argv,
-						 gboolean has_master,
 						 GError **error);
-	BraseroBurnResult      (*post)        (BraseroProcess *process,
-					         BraseroBurnResult retval);
+
+	/* since burn-process.c doesn't know if it should call finished_session
+	 * of finished track this allows to override the default call which is
+	 * brasero_job_finished_track */
+	BraseroBurnResult      	(*post)       	(BraseroJob *job);
 } BraseroProcessClass;
 
 GType brasero_process_get_type();
 
+/**
+ * This function allows to set an error that is used if the process doesn't 
+ * return 0.
+ */
+void
+brasero_process_deferred_error (BraseroProcess *process,
+				GError *error);
+
 #endif /* PROCESS_H */
