@@ -34,6 +34,7 @@
 #include "burn-caps.h"
 #include "burn-session.h"
 #include "burn-task.h"
+#include "burn-medium.h"
 
 G_BEGIN_DECLS
 
@@ -44,11 +45,8 @@ G_BEGIN_DECLS
 #define BRASERO_IS_BURN_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), BRASERO_TYPE_BURN))
 #define BRASERO_BURN_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), BRASERO_TYPE_BURN, BraseroBurnClass))
 
-typedef struct BraseroBurnPrivate BraseroBurnPrivate;
-
 typedef struct {
 	GObject parent;
-	BraseroBurnPrivate *priv;
 } BraseroBurn;
 
 typedef struct {
@@ -56,14 +54,18 @@ typedef struct {
 
 	/* signals */
 	BraseroBurnResult		(*insert_media_request)		(BraseroBurn *obj,
-									 BraseroMediaType error,
-									 BraseroMediaType required_media);
+									 BraseroBurnError error,
+									 BraseroMedia required_media);
 
 	BraseroBurnResult		(*ask_disable_joliet)		(BraseroBurn *obj);
 
 	BraseroBurnResult		(*warn_data_loss)		(BraseroBurn *obj);
+	BraseroBurnResult		(*warn_previous_session_loss)	(BraseroBurn *obj);
+	BraseroBurnResult		(*warn_audio_to_appendable)	(BraseroBurn *obj);
 	BraseroBurnResult		(*warn_rewritable)		(BraseroBurn *obj);
 
+	BraseroBurnResult		(*dummy_success)		(BraseroBurn *obj);
+
 	void				(*progress_changed)		(BraseroBurn *obj,
 									 gdouble overall_progress,
 									 gdouble action_progress,
@@ -77,34 +79,30 @@ BraseroBurn *brasero_burn_new ();
 
 BraseroBurnResult 
 brasero_burn_record (BraseroBurn *burn,
-		     BraseroBurnFlag flags,
-		     NautilusBurnDrive *drive,
-		     gint speed,
-		     const BraseroTrackSource *source,
-		     const gchar *output,
+		     BraseroBurnSession *session,
 		     GError **error);
 
 BraseroBurnResult
-brasero_burn_blank (BraseroBurn *burn,
-		    BraseroBurnFlag flags,
-		    NautilusBurnDrive *drive,
-		    gboolean fast,
+brasero_burn_check (BraseroBurn *burn,
+		    BraseroBurnSession *session,
 		    GError **error);
 
 BraseroBurnResult
-brasero_burn_cancel (BraseroBurn *burn, gboolean protect);
+brasero_burn_blank (BraseroBurn *burn,
+		    BraseroBurnSession *session,
+		    GError **error);
+
+BraseroBurnResult
+brasero_burn_cancel (BraseroBurn *burn,
+		     gboolean protect);
 
 BraseroBurnResult
 brasero_burn_status (BraseroBurn *burn,
-		     NautilusBurnMediaType *media,
+		     BraseroMedia *info,
 		     gint64 *isosize,
 		     gint64 *written,
 		     gint64 *rate);
 
-void
-brasero_burn_set_session (BraseroBurn *burn,
-			  BraseroBurnSession *session);
-
 void
 brasero_burn_get_action_string (BraseroBurn *burn,
 				BraseroBurnAction action,
