@@ -243,7 +243,7 @@ on_about_cb (GtkAction *action, BraseroApp *app)
                 N_("You should have received a copy of the GNU General "
 		   "Public License along with Brasero; if not, write "
 		   "to the Free Software Foundation, Inc., "
-                   "59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"),
+                   "51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA"),
 		NULL
         };
 
@@ -258,10 +258,10 @@ on_about_cb (GtkAction *action, BraseroApp *app)
 			     NULL);
 
 	gtk_show_about_dialog (GTK_WINDOW (app->mainwin),
-			       "name", "Brasero",
+			       "program-name", "Brasero",
 			       "comments", comments,
 			       "version", VERSION,
-			       "copyright", "Copyright © 2005-2007 Philippe Rouquier",
+			       "copyright", "Copyright © 2005-2008 Philippe Rouquier",
 			       "authors", authors,
 			       "documenters", documenters,
 			       "website", "http://www.gnome.org/projects/brasero",
